#include <curl/curl.h>
#include <sstream>
#include <string>
#include <iostream>
#include <vector>
#include "histogram.h"
#include "svg.h"
using namespace std;

struct Input {
    vector<double> numbers;
    size_t bin_count{};
};

Input
input_data(istream& in, bool prompt){
    size_t number_count;
    if (prompt == true) cerr << "Enter number_count: ";
    in >> number_count;

    Input in1;
    in1.numbers.resize(number_count);
    if (prompt == true) cerr << "Enter numbers: ";
    for (size_t i = 0; i < number_count; i++){
        in >> in1.numbers[i];
    }

    if (prompt == true) cerr << "Enter bin count: ";
    in >> in1.bin_count;

    return in1;
}

static size_t
write_data(void* items, size_t item_size, size_t item_count, void* ctx){
    size_t data_size = item_size * item_count;
    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    buffer->write(reinterpret_cast<const char*>(items), data_size);
    return data_size;
}

Input
download(const string& address) {
    stringstream buffer;

    curl_global_init(CURL_GLOBAL_ALL);
    CURL *curl = curl_easy_init();
    if(curl) {
        CURLcode res;
        curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
        res = curl_easy_perform(curl);
        curl_easy_cleanup(curl);
        if (res != CURLE_OK) {
            fprintf(stderr, "curl_easy_perform() failed: %s\n", curl_easy_strerror(res));
            exit(1);
        }
    }

    return input_data(buffer, false);
}

int
main(int argc, char* argv[])
{
    Input input;
    if (argc > 1) {
        input = download(argv[1]);
    } else {
        input = input_data(cin, true);
    }

    const auto bins = make_histogram(input.numbers, input.bin_count);
    show_histogram_svg(bins);
    return 0;
}
